import {
  Modal, ModalService, Select, GridEditable, Iterable, Button,
  AlertService, DateRange, Grid, Text, Textarea,
} from '@zeedhi/common';
import {
  Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';

import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

import { getRouteParams } from '../router';

export class ApontamentoController {
  public dataSelects: IDictionary = {
    selectUsuario: [],
    selectSetor: [],
  };

  public parametroBusca: IDictionary = [];

  public apontamentos: IDictionary = [];

  public valorHora: String = '';

  public dataReferencia: String = '';

  public textoModal: String = '';

  public nrfuncionario: Number = 0;

  public nrapontamento: Number = 0;

  public dataFilter: IDictionary = {
    NRFUNCIONARIO: '',
    NRSETOR: '',
    DTULTATU: [],
  };

  private modalApontamento!: Modal;

  private modalDetalhes!: Modal;

  public async criarModalApontamento() {
    this.modalApontamento = await ModalService.createFromJson('./modal/modalNovoApontamento', true);
  }

  public async criarModalDetalhes() {
    this.modalDetalhes = await ModalService.createFromJson('./modal/modalDetalhesApontamento', true);
  }

  public destruirModalApontamento() {
    this.modalApontamento?.destroy();
  }

  public destroyModalDetalhes() {
    this.modalDetalhes?.destroy();
  }

  public abrirModalApontamento() {
    this.modalApontamento.show();
  }

  public abrirModalDetalhes({ row }: IEventParam<Button>) {
    this.nrapontamento = row.id.split('-');
    this.modalApontamento.destroy();
    this.modalDetalhes.show();
    this.criarModalApontamento();
  }

  public fecharModalApontamento() {
    this.modalApontamento.hide();
    this.modalApontamento.destroy();
    this.criarModalApontamento();
  }

  public fecharModalDetalhes() {
    this.modalDetalhes.hide();
    this.modalDetalhes.destroy();
    this.criarModalDetalhes();
  }

  public carregaDadosFiltros() {
    this.getDadosUsuario();
    this.getDadosSetor();
  }

  public setDataSelects({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  public setData(nm: string, id: number) {
    return {
      name: nm,
      value: id,
    };
  }

  public getDadosUsuario() {
    Http.get('/Employee').then((response: IDictionary) => {
      response.data.dataset.employee.forEach((dados: IDictionary) => {
        this.dataSelects.selectUsuario.push(this.setData(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
      });
    });
  }

  public getDadosSetor() {
    Http.get('/getSector').then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        this.dataSelects.selectSetor.push(this.setData(dados.NMSETOR, dados.NRSETOR));
      });
    });
  }

  public getDadosCardApontamento() {
    const selectObjectItareble = Metadata.getInstance<IterableComponentRender>('iterable');
    Http.post('/getDadosApontamentos', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        selectObjectItareble.datasource.data.push({
          id: dados.NRFUNCIONARIO,
          name: dados.NMFUNCIONARIO,
          data: dados.ULTATUALIZACAO,
        });
      });
    });
  }

  public getParamURL() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    const nrfuncionario = parseInt(nr, 10);
    this.nrfuncionario = nrfuncionario;
  }

  public getDadosFiltros({ component }: IEventParam<any>) {
    if (component.name === 'selectUsuario') {
      this.dataFilter.NRFUNCIONARIO = component.value;
      this.reloadItarable();
      this.getDadosCardApontamento();
    } else if (component.name === 'selectSetor') {
      this.dataFilter.NRSETOR = component.value;
      this.reloadItarable();
      this.getDadosCardApontamento();
    } else {
      this.dataFilter.DTULTATU = component.value;
      this.reloadItarable();
      this.getDadosCardApontamento();
    }
  }

  public reloadItarable() {
    const selectObjectItareble = Metadata.getInstance<IterableComponentRender>('iterable');
    selectObjectItareble.reload();
  }

  public recarregarGridModalNovoApontamento({ component }: IEventParam<Button>) {
    (component.parent as Iterable).reload();
  }

  public geraConteudoGrid() {
    const data = Metadata.getInstance<DateRange>('periodo').value;
    const selectObjectGrid = Metadata.getInstance<Grid>('grid');
    this.apontamentos = [];
    selectObjectGrid.reload();
    if (data.length > 1) {
      const dtInicio = this.criarNovoObjetoData(data[0]);
      const dtFim = this.criarNovoObjetoData(data[1]);
      let novaData = dtInicio;
      const NRFUNCIONARIO = this.nrfuncionario;
      this.formataData(novaData);
      let contador = 1;
      while (novaData < dtFim) {
        novaData = new Date(novaData.getTime() + (24 * 60 * 60 * 1000 - 1));
        this.apontamentos.push({
          id: contador,
          data: this.formataData(novaData),
          funcionario: this.nrfuncionario,
          entrada: '',
          intervalo: '',
          saida: '',
          hrs_trabalhada: '00:00',
          ausencia: '00:00',
          horas_totais: '00:00',
          falta: '-',
          obs: false,
        });
        contador += 1;
      }
      Http.post('/buscaRegistrosHora', { NRFUNCIONARIO }).then((dados: IDictionary) => {
        const registros = dados.data.dataset.dados;
        registros.forEach((registrosApontamentos: IDictionary) => {
          Object.keys(this.apontamentos).forEach((t: any) => {
            if (this.apontamentos[t].data === registrosApontamentos.DATA) {
              if (registrosApontamentos.DSOBSREGISTRO !== null) {
                this.apontamentos.splice(this.apontamentos[t], this.apontamentos[t] = {
                  id: `${registrosApontamentos.NRAPONTAMENTO}-${registrosApontamentos.DATA}`,
                  data: registrosApontamentos.DATA,
                  funcionario: this.nrfuncionario,
                  entrada: registrosApontamentos.HRENTRADA,
                  intervalo: registrosApontamentos.HRINTERVALO,
                  saida: registrosApontamentos.HRSAIDA,
                  hrs_trabalhada: registrosApontamentos.HRTRABALHADAS,
                  ausencia: registrosApontamentos.HRAUSENCIA,
                  horas_totais: registrosApontamentos.HRTOTAIS,
                  falta: registrosApontamentos.IDFALTA,
                  obs: true,
                });
              } else {
                this.apontamentos.splice(this.apontamentos[t], this.apontamentos[t] = {
                  id: `${registrosApontamentos.NRAPONTAMENTO}-${registrosApontamentos.DATA}`,
                  data: registrosApontamentos.DATA,
                  funcionario: this.nrfuncionario,
                  entrada: registrosApontamentos.HRENTRADA,
                  intervalo: registrosApontamentos.HRINTERVALO,
                  saida: registrosApontamentos.HRSAIDA,
                  hrs_trabalhada: registrosApontamentos.HRTRABALHADAS,
                  ausencia: registrosApontamentos.HRAUSENCIA,
                  horas_totais: registrosApontamentos.HRTOTAIS,
                  falta: registrosApontamentos.IDFALTA,
                  obs: false,
                });
              }
            }
          });
        });
        selectObjectGrid.datasource.data = this.apontamentos as any;
      });
    }
  }

  public criarNovoObjetoData(data: string) {
    const arr = data.split('/');
    const invertePosicao = arr.reverse();
    const formata = invertePosicao.join('/');
    const dataForm = new Date(formata);
    return dataForm;
  }

  public formataData(novaData: Date) {
    let dataFormatada = '';
    if (novaData.getDate() < 10) {
      const diaFormatado = `0${novaData.getDate()}`;
      if ((novaData.getMonth() + 1) < 10) {
        const mesFormatado = `0${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      } else {
        const mesFormatado = `${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      }
    } else {
      const diaFormatado = novaData.getDate();
      if ((novaData.getMonth() + 1) < 10) {
        const mesFormatado = `0${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      } else {
        const mesFormatado = `${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      }
    }
    return dataFormatada;
  }

  public linhaClicada({ component }: any) {
    this.dataReferencia = component.datasource.currentRow.data;
    if (component.datasource.currentRow.entrada === '') {
      this.parametroBusca = [];
      this.parametroBusca.push({
        NRAPONTAMENTO: -1,
        DATA: component.datasource.currentRow.data,
      });
      this.abrirModalApontamento();
    } else {
      const nr = component.datasource.currentRow.id.split('-');
      this.parametroBusca = [];
      this.parametroBusca.push({
        NRAPONTAMENTO: nr[0],
        DATA: component.datasource.currentRow.data,
      });
      this.abrirModalApontamento();
    }
  }

  public async getDadosApontamentoInserido() {
    const selectObjectGrid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    selectObjectGrid.datasource.data = [];
    await Http.post('/getDadosApontamentoInserido', this.parametroBusca).then((response: IDictionary) => {
      let valor = '';
      let tipo = '';
      let dt = '';
      const { dados } = response.data.dataset;
      dados.forEach((t: any) => {
        if (t.DTHRSAIDA === null && t.IDALMOCO === 'N') {
          if (t.DTHRENTRADA !== null && t.IDALMOCO === 'N') {
            valor = t.DTHRENTRADA.slice(11, 17);
            dt = t.DTHRENTRADA.slice(0, 10);
            tipo = '1';
          }
        } else if (t.DTHRSAIDA === null && t.IDALMOCO === 'S') {
          if (t.DTHRENTRADA !== null && t.IDALMOCO === 'S') {
            valor = t.DTHRENTRADA.slice(11, 17);
            dt = t.DTHRENTRADA.slice(0, 10);
            tipo = '2';
          }
        } else if (t.DTHRSAIDA !== null && t.IDALMOCO === 'S') {
          if (t.DTHRENTRADA === null && t.IDALMOCO === 'S') {
            valor = t.DTHRSAIDA.slice(11, 17);
            dt = t.DTHRSAIDA.slice(0, 10);
            tipo = '3';
          }
        } else if (t.DTHRSAIDA !== null && t.IDALMOCO === 'N') {
          if (t.DTHRENTRADA === null && t.IDALMOCO === 'N') {
            valor = t.DTHRSAIDA.slice(11, 17);
            dt = t.DTHRSAIDA.slice(0, 10);
            tipo = '4';
          }
        }
        const newRow = {
          id: t.NRREGISTROHORAS,
          enable_edit: false,
          name: t.NRREGISTROHORAS,
          tipoEntrada: tipo,
          idAlmoco: t.IDALMOCO,
          data: dt,
          funcionario: this.nrfuncionario,
          hora: valor,
          buttonSave: t.NRREGISTROHORAS,
          btnDelete: t.NRREGISTROHORAS,
        };
        selectObjectGrid.addNewRow(newRow);
      });
    });
    this.mostraTotalHorasModal();
    this.controlaVisibilidadeApurar();
  }

  public async salvarApontamento({ row }: IEventParam<Button>) {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    await grid.saveEditedRows();
    grid.datasource.data.forEach((objeto: any) => {
      if (objeto.name === row.name) {
        Http.post('/insertNovoApontamento', { objeto }).then((response: any) => {
          if (response.data.dataset.dados.retorno === 'OK') {
            const texto = 'Apontamento inserido com sucesso!';
            const sucesso = 'success';
            this.mensagemAlert(texto, sucesso);
          } else {
            const texto = 'Ocorreu um erro ao inserir o apontamento';
            const sucesso = 'error';
            this.mensagemAlert(texto, sucesso);
          }
          this.geraPrimeiroUltimoDiaMes();
          this.geraConteudoGrid();
          this.getDadosApontamentoInserido();
        });
      }
    });
  }

  public buttonVisible = true;

  public editarLinha({ row }: IEventParam<GridEditable>) {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    const editedRows = grid.getEditedRows();
    const conditionsRow = editedRows.find((editedRow) => editedRow.name === row.name) || row;
    conditionsRow.enable_edit = true;
    this.buttonVisible = true;
    grid.reapplyConditions(conditionsRow);
  }

  public testRowIsEditable({ row }: { row: IDictionary<any> }) {
    return row.enable_edit;
  }

  public addNovaBatida({ component }: any) {
    const grid: GridEditable = component.parent;
    const newId = grid.datasource.data.length;
    const newRow = {
      id: `${newId}`,
      enable_edit: true,
      name: `${newId}`,
      tipoEntrada: '',
      data: this.dataReferencia,
      funcionario: this.nrfuncionario,
      hora: '',
    };
    grid.addNewRow(newRow);
  }

  public mensagemAlert(text: string, tipo: string) {
    AlertService.show({
      name: 'basic-alert',
      text: `${text}`,
      top: true,
      left: false,
      timeout: 3000,
      color: `${tipo}`,
    });
  }

  public deleteLinha({ row }: any) {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    grid.datasource.data.forEach((objeto: any) => {
      if (objeto.tipoEntrada === '') {
        grid.datasource.data.splice(objeto, 1);
      } else if (objeto.tipoEntrada !== '') {
        if (objeto.name === row.name) {
          Http.post('deleteRegistroHoras', { objeto }).then((response: any) => {
            if (response.data.dataset.dados.retorno === 'DELETADO') {
              const texto = 'Apontamento deletado com sucesso!';
              const sucesso = 'success';
              this.mensagemAlert(texto, sucesso);
              this.geraPrimeiroUltimoDiaMes();
              this.geraConteudoGrid();
              grid.datasource.data = [];
              this.getDadosApontamentoInserido();
            } else {
              const texto = 'Ocorreu um erro ao deletar o apontamento';
              const sucesso = 'error';
              this.mensagemAlert(texto, sucesso);
            }
          });
        }
      }
    });
  }

  public apurar() {
    const parametrosUpdate: IDictionary = [];
    let entrada = '';
    let saida = '';
    let entradaAlmoco = '';
    let saidaAlmoco = '';
    let intervalo = '';
    let horasTrabalhadas = '';
    let horasTotais = '';
    let dataReferencia = '';
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    grid.datasource.data.forEach((dados: IDictionary) => {
      if (dados.tipoEntrada === '1' && dados.idAlmoco === 'N') {
        entrada = dados.hora;
        dataReferencia = dados.data;
      } else if (dados.tipoEntrada === '4' && dados.idAlmoco === 'N') {
        saida = dados.hora;
      } else if (dados.tipoEntrada === '2' && dados.idAlmoco === 'S') {
        entradaAlmoco = dados.hora;
      } else if (dados.tipoEntrada === '3' && dados.idAlmoco === 'S') {
        saidaAlmoco = dados.hora;
      }
    });
    intervalo = this.calculoIntervalo(entradaAlmoco, saidaAlmoco);
    horasTrabalhadas = this.calculoHoraTotal(entrada, saida, intervalo);
    horasTotais = this.calculoHoraTotal(entrada, saida, intervalo);
    parametrosUpdate.push({
      DATA: dataReferencia,
      NRFUNCIONARIO: this.nrfuncionario,
      HRENTRADA: entrada,
      HRSAIDA: saida,
      HRINTERVALO: intervalo,
      HRTRABALHADAS: horasTrabalhadas,
      HRAUSENCIA: '00:00',
      HRTOTAIS: horasTotais,
      IDFALTA: 'N',
      IDAPURADO: 'S',
    });
    Http.post('/apurarHoras', parametrosUpdate).then((response: any) => {
      if (response.data.dataset.dados.retorno === 'APURADO') {
        const texto = 'Registro apurado com sucesso!';
        const sucesso = 'success';
        this.mensagemAlert(texto, sucesso);
      } else {
        const texto = 'Ocorreu um erro ao apurar o registro';
        const sucesso = 'error';
        this.mensagemAlert(texto, sucesso);
      }
      this.geraPrimeiroUltimoDiaMes();
      this.geraConteudoGrid();
    });
  }

  public calculoIntervalo(entradaAlmoco: string, saidaAlmoco: string) {
    const h1 = entradaAlmoco.split(':');
    const h2 = saidaAlmoco.split(':');

    const EntrHora = Number(h1[0]);
    const EntrMin = Number(h1[1]);
    const SaiHora = Number(h2[0]);
    const SaiMin = Number(h2[1]);

    const entrada = this.minutos(EntrHora, EntrMin);
    const saida = this.minutos(SaiHora, SaiMin);

    const intervalo = this.formataIntervalo(entrada, saida);
    return intervalo;
  }

  public minutos(h: number, m: number) {
    const hora = h * 3600;
    const minutos = m * 60;
    return {
      h: hora,
      m: minutos,
    };
  }

  public formataIntervalo(entrada: any, saida: any) {
    const hora = ((saida.h - entrada.h) / 3600);
    const minuto = ((saida.m - entrada.m) / 60);
    let retorno = '';
    if (minuto <= 10) {
      retorno = `0${hora}:0${minuto}`;
    } else {
      retorno = `${hora}:${minuto}`;
    }
    return retorno;
  }

  public calculoHoraTotal(entrada: string, saida: string, intervalo: string) {
    const h1 = entrada.split(':');
    const h2 = saida.split(':');
    const inter = intervalo.split(':');

    const EntrHora = Number(h1[0]);
    const EntrMin = Number(h1[1]);
    const SaiHora = Number(h2[0]);
    const SaiMin = Number(h2[1]);
    const interHora = Number(inter[0]);
    const interMIn = Number(inter[1]);

    const inicio = this.minutos(EntrHora, EntrMin);
    const fim = this.minutos(SaiHora, SaiMin);
    const interv = this.minutos(interHora, interMIn);

    const total = this.formataHoraTotal(inicio, fim, interv);
    return total;
  }

  public formataHoraTotal(entrada: any, saida: any, intervalo: any) {
    const result = (saida.h - intervalo.h);
    const hora = ((result - entrada.h) / 3600);
    const result1 = (saida.m - intervalo.m);
    const minuto = ((result1 - entrada.m) / 60);
    let retorno = '';
    if (hora < 10) {
      retorno = `0${hora}:0${Math.abs(minuto)}`;
      if (Math.abs(minuto) < 10) {
        retorno = `0${hora}:0${Math.abs(minuto)}`;
      } else {
        retorno = `0${hora}:${Math.abs(minuto)}`;
      }
    } else {
      retorno = `${hora}:${Math.abs(minuto)}`;
      if (Math.abs(minuto) < 10) {
        retorno = `${hora}:0${Math.abs(minuto)}`;
      } else {
        retorno = `${hora}:${Math.abs(minuto)}`;
      }
    }
    return retorno;
  }

  public VisibilidadeInformacao({ row }: any) {
    return row.obs;
  }

  public geraPrimeiroUltimoDiaMes() {
    const date = new Date();
    const primeiroDia = new Date(date.getFullYear(), date.getMonth(), 1);
    const ultimoDia = new Date(date.getFullYear(), date.getMonth() + 1, 0);
    Metadata.getInstance<DateRange>('periodo').value = [this.formataData(primeiroDia), this.formataData(ultimoDia)];
  }

  public mostraTotalHorasModal() {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    if (grid.datasource.data.length === 4) {
      let entrada = '';
      let saida = '';
      let intervalo = '';
      let horasTotais = '';
      let entradaAlmoco = '';
      let saidaAlmoco = '';
      grid.datasource.data.forEach((dados: IDictionary) => {
        if (dados.tipoEntrada === '1' && dados.idAlmoco === 'N') {
          entrada = dados.hora;
        } else if (dados.tipoEntrada === '4' && dados.idAlmoco === 'N') {
          saida = dados.hora;
        } else if (dados.tipoEntrada === '2' && dados.idAlmoco === 'S') {
          entradaAlmoco = dados.hora;
        } else if (dados.tipoEntrada === '3' && dados.idAlmoco === 'S') {
          saidaAlmoco = dados.hora;
        }
      });

      intervalo = this.calculoIntervalo(entradaAlmoco, saidaAlmoco);
      horasTotais = this.calculoHoraTotal(entrada, saida, intervalo);
      Metadata.getInstance<Text>('textModalNovoApontamento').text = `Total: ${horasTotais}`;
    }
  }

  public controlaVisibilidadeApurar() {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    if (grid.datasource.data.length === 4) {
      Metadata.getInstance<Button>('apurarNovoApontamento').disabled = false;
    } else {
      Metadata.getInstance<Button>('apurarNovoApontamento').disabled = true;
    }
  }

  public getDadosObservacao() {
    let observacao = '';
    Http.post('/getDadosDocEObservacoes', this.nrapontamento).then((response: any) => {
      response.data.dataset.dados.forEach((dados: any) => {
        observacao = observacao.concat(dados.DSOBSREGISTRO, ',');
        Metadata.getInstance<Textarea>('textarea').value = `${observacao}`;
      });
    });
  }

  public insereTextoModal() {
    this.textoModal = `Intervalo de Marcação - ${this.dataReferencia}`;
  }
}
