import {
  IDictionary, Http, Metadata,
  IEventParam, Loader, Router,
  SimpleDatasource,
} from '@zeedhi/core';
import {
  Select, Container, Text, Card, Textarea, DialogService,
  Date, CheckboxMultiple, IComponentRender, Button, AlertService,
  Form, TextInput, FileInput, Switch,
} from '@zeedhi/common';
import {
  IterableCarousel, CardTitle,
} from '@zeedhi/teknisa-components-common';
import { getRouteParams } from '../router';

export class VistoriasController {
  public hierarquiasController = Loader.getInstance('HierarquiasController');

  public modalController = Loader.getInstance('NovoCriterioController');
  /* ---------------------- Tela Vistorias  --------------------- */

  // Variável que abriga o cards atividades
  public selected: IDictionary = [];

  public nomeCardClicado: string = '';

  public inspectionName: string = '';

  public quantidadeAtividades: Number = 0;

  // Array com os valores de cada select
  public dataSelects: IDictionary = {
    inspectionLocalType: [],
    inspectionLocals: [],
    inspectionChecklists: [],
    inspectionEmployees: [],
  };

  public dataBase: IDictionary = [];

  // Recebe cada info do banco e retorna para o array cima
  public setData(nm: string, id: number) {
    return {
      name: nm,
      value: id,
    };
  }

  // Inicializa as funções get
  public initializeData() {
    this.setLocalType();
    this.setLocals();
    this.setChecklists();
    this.setEmployees();
  }

  // Seta em cada select os valores do array dataSelects
  public setDataSelects({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  // Busca os "Tipos de Local"
  public setLocalType() {
    Http.get('/getLocalType').then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        this.dataSelects.inspectionLocalType.push(this.setData(database.NMTIPOLOCALCLI, database.NRTIPOLOCALCLI));
      });
    });
  }

  // Busca os "Locais de Atendimento"
  public setLocals() {
    Http.get('/getLocals').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        this.dataSelects.inspectionLocals.push(this.setData(database.NMLOCALCLIENTE, database.NRLOCALCLIENTE));
      });
    });
  }

  // Busca os "Checklists"
  public setChecklists() {
    Http.get('/getCheckListLocalAtend').then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        this.dataSelects.inspectionChecklists.push(this.setData(database.NMCHECKLIST, database.NRCHECKLIST));
      });
    });
  }

  // Busca Funcionários (Usando uma rota temporária, depois trocar para getUsers de UsuariosController)
  public async setEmployees() {
    const hierarchyUsersList: IDictionary = await this.hierarquiasController.mountHierarchyUsersArray();
    Http.get(`/getEmployee?params=${hierarchyUsersList}`).then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        this.dataSelects.inspectionEmployees.push(this.setData(database.NMFUNCIONARIO, database.NRFUNCIONARIO));
      });
    });
  }

  // Monta e exibe os cards com as informações de cada vistoria
  public async mountInpectionsCards(selectsFilterInfo: IDictionary = {}) {
    const hierarchyUsersList: IDictionary = await this.hierarquiasController.mountHierarchyUsersArray();
    const containerObject = Metadata.getInstance<Container>('inpectionsListContainer');
    containerObject.children = [];

    Http.post('/getInspectionsInfo', { params: selectsFilterInfo, params2: hierarchyUsersList }).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        containerObject.children.push(< unknown > this.inspectionCard(database.NRGERENCIAMENTOCHK, database.NMGERENCIAMENTOCHK, database.NRCHECKLISTAGEND, database.NMLOCALCLIENTE, database.NMLOCALPAI, database.DTHORAAGEND, database.NMFUNCIONARIO, database.IDSITUACAOGERCHK, database.IDSITUACAOAGEND) as IComponentRender);
      });
    });
  }

  // Manuseia a data vinda do banco
  public timeManager(time: any) {
    const date = `${time[0].split('-')[2]}-${time[0].split('-')[1]}-${time[0].split('-')[0]} - ${time[1].substr(0, 5)}`;
    return date;
  }

  // Seta as cores de Situação
  public statusColor({ component }: IEventParam<Text>) {
    if (component.text === 'Aguardando Vistoria') {
      component.text = "<span  style='color: #ff6700; font-size: 14px;'>AGUARDANDO VISTORIA</span>";
    } else if (component.text === 'Em andamento') {
      component.text = "<span  style='color: #023e8a; font-size: 14px;'>EM ANDAMENTO</span>";
    } else if (component.text === 'Avaliado') {
      component.text = "<span  style='color: #2b9348; font-size: 14px;'>A V A L I A D O</span>";
    } else if (component.text === 'Tarefa Pendente') {
      component.text = "<span style='color: #ff6700; font-size: 14px;'>TAREFA PENDENTE</span>";
    } else {
      component.text = "<span style='color: #FA980D; font-size: 14px;'>CANCELADO</span>";
    }
  }

  // Início do eventro de filtro e exibição das vistorias
  public initializeFilter() {
    this.mountInpectionsCards(this.filter());
  }

  // Adiciona as informações dos selects em um array
  public filter() {
    const dateObject = Metadata.getInstance<Date>('inspectionDate').value;
    return {
      'LC.NRTIPOLOCALCLI': Metadata.getInstance<Select>('inspectionLocalType').value,
      'LC.NRLOCALCLIENTE': Metadata.getInstance<Select>('inspectionLocals').value,
      'GCHK.NRCHECKLIST': Metadata.getInstance<Select>('inspectionChecklists').value,
      'GCHK.NRFUNCIONARIO': Metadata.getInstance<Select>('inspectionEmployees').value,
      'GCHK.IDSITUACAOGERCHK': Metadata.getInstance<Select>('inspectionStatus').value,
      'TO_CHAR(GCHK.DTHORAAGEND,\'DD/MM/YYYY\')': ((dateObject != null) ? `TO_CHAR('${dateObject}')` : null),
    };
  }

  // Card de Vistorias
  public inspectionCard(nrgerenciamentochk: number, nmgerenciamentochk: string, nrchecklistagend: number, nmlocalcliente: string, nmlocalclientepai: string, dthoraagend: string, nmfuncionario: string, idsituacaogerchk: number, idsituacaoagend: number) {
    return {
      name: 'ColunaCard',
      component: 'ZdCol',
      cssClass: 'zd-pa-0 spacing-between-cards',
      children: [
        {
          name: `inspection${nrgerenciamentochk}`,
          component: 'ZdCard',
          to: `/vistorias/detalhes/${nrgerenciamentochk}`,
          events: {
            click: '{{VistoriasController.setInspectionToUpdate}}',
          },
          children: [
            {
              name: nmlocalcliente,
              component: 'ZdText',
              text: '',
              isVisible: false,
            },
            {
              name: nrchecklistagend,
              component: 'ZdText',
              text: '',
              isVisible: false,
            },
            {
              name: 'linhaInterna',
              component: 'ZdRow',
              justify: 'space-between',
              align: 'center',
              children: [
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: nmgerenciamentochk,
                      tag: 'h4',
                      cssClass: 'color-title-card',
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: 3,
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-map-marker-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: `${nmlocalcliente} | ${nmlocalclientepai}`,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-clock-check-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: dthoraagend,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '3',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-account-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: nmfuncionario,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: 2,
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: idsituacaogerchk,
                          style: {
                            'font-size': '15px',
                          },
                          events: {
                            onCreated: '{{VistoriasController.statusColor}}',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: idsituacaoagend,
                          isVisible: false,
                          style: {
                            'font-size': '15px',
                          },
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  /* ---------------------- Tela vistoriasDetalhes  --------------------- */

  public isVisibleElement: boolean = !(localStorage.getItem('idpermisssvistoria') === 'V');

  public isDisabledElement: boolean = (localStorage.getItem('idpermisssvistoria') === 'V');

  // Fecha a caixa de diálogo
  private dialogHideEvents: any = {
    click: () => {
      DialogService.hide();
    },
  };

  // Chama o evento de clear data e fecha a caixa de diálogo
  private dialogConfirmEvents: any = {
    click: () => {
      this.clearInspectionData();
      DialogService.hide();
    },
  };

  // Chama o evento de save inspection e fecha a caixa de diálogo
  // private saveConfirmEvents: any = {
  //   click: () => {
  //     this.saveInspectionDetails(Metadata.getInstance<Button>('saveInspectionDt'));
  //     DialogService.hide();
  //   },
  // };

  // Confirma que quer sair da página
  private finishConfirmEvents: any = {
    click: () => {
      this.finishInspectionDetails();
      DialogService.hide();
    },
  };

  // Recebe o nome e id da atividade
  public setActivityData(ActName: string, ActId: number) {
    return {
      id: ActId,
      name: ActName,
    };
  }

  // Recebe o nome, id e idmarcado do critério
  public setCriterioData(nm: string, id: number, markedId: string) {
    if (markedId === 'S') Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.push(id);
    return {
      name: nm,
      value: id,
      criterioId: markedId,
    };
  }

  public changeCardColor({ component }: IEventParam<Card>) {
    if (component.cssClass === '') {
      component.cssClass = 'inspectionCardClicked';
      component.children[0].color = '#ffffff';

      // const id = component.name.substr(8);
      // const subtitle = component.children[1].text;

      // this.getCriteria(id, subtitle);
    } else { // Se tirar o clicar pela segunda vez tirar a seleção
      component.cssClass = '';
      component.children[0].color = '#0077b6';

      const checkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
      checkObject!.datasource!.data = [];
    }
  }

  // Busca nome da Vistoria e Onde é feita
  public getInspectionNameAndLocal(nrgerenciamentochk: string) {
    Http.get(`/getInspectionInfo/${nrgerenciamentochk}/nameAndLocal`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        Metadata.getInstance<Text>('inspectionDtName').text = database.NMGERENCIAMENTOCHK;
        Metadata.getInstance<Text>('inspectionDtLocal').text = database.NMLOCALCLIENTE;
      });
    });
  }

  // Busca critérios
  public getCriteria(id:string, subtitle: string) {
    const params = id;
    const checkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const cardObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const btnObject = Metadata.getInstance<Button>('newCriterioinspectionDt');
    cardObject.subtitle = subtitle;
    checkObject!.datasource!.data = [];
    checkObject!.value = [];
    Http.get(`/getCriteria/${params}`).then((response: IDictionary) => {
      btnObject.disabled = (response.data.dataset.info[0].IDSITUACAOAGEND !== '4');
      response.data.dataset.info.forEach((database: IDictionary) => {
        checkObject?.datasource?.data?.push(this.setCriterioData(database.NMCRITERIOTARGERCHK, database.NRCRITERIOTARGERCHK, database.IDMARCADO));
      });
    });
  }

  // Busca as atividades
  public getActivities(id: string) {
    const cardObject = Metadata.getInstance<IterableCarousel>('inspectionDtActivities');
    Http.get(`/getActivities/${id}/Activities`).then((response: IDictionary) => {
      this.quantidadeAtividades = response.data.dataset.info.length;
      response.data.dataset.info.forEach((database: IDictionary) => {
        cardObject.iterable.datasource.data.push(this.setActivityData(database.NMATIVIDADE, database.NRATIVIDADECHK));
      });
    });
  }

  // Método para enviar os dados para update da vistoria.
  public saveSurvey() {
    const dataBase = Object.keys(this.dataBase).map((key: string) => this.dataBase[key]);
    Http.post('/updateInspection', { dataBase }).then((response: IDictionary) => {
      const resposta = response.data.dataset.info[0];
      if (resposta === 'OK') {
        AlertService.show({
          name: 'buttons-alert',
          text: 'Vistoria realizada com sucesso!',
          color: 'success',
          top: true,
          left: false,
          timeout: 1500,
        });
      } else {
        DialogService.show({
          name: 'warning-dialog',
          type: 'warning',
          title: 'Erro!',
          text: 'Operação bloqueada! Ocorreu um erro durante a finalização da vistoria, portando esta vistoria não foi finalizada.<br><br>Se o problema persistir entre em contato com o suporte.',
        });
      }
    });
  }

  // Monta os critérios de não conformidades na tela antes de finalizar a tarefa
  public getNoteAndCriteria({ component }: IEventParam<Card>) {
    if (Object.keys(this.selected).length) {
      this.dataBase[this.selected.name] = {
        NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
        NRATIVIDADECHK: this.selected.name.substr(8),
        valuesChkbox: Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.map((dados: any) => `${dados}`).join(','),
        valueObservation: Metadata.getInstance<Textarea>('inspectionDtNote').value,
        valueSwitch: Metadata.getInstance<Switch>('basicUsageSwitch').value,
      };
      this.selected.cssClass = 'inspectionCard';
    }
    component.cssClass = 'inpectionCardClicked';
    this.selected = component;
    const textAreaObject = Metadata.getInstance<Textarea>('inspectionDtNote');
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage');
    const criteriaTitleObject = Metadata.getInstance<Text>('inspectionDtCriteriaTitle');
    const cardObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const btnObject = Metadata.getInstance<Button>('newCriterioinspectionDt');
    cardObject.subtitle = component.children[1].text;
    textAreaObject.disabled = this.isDisabledElement;
    fileInputObject.disabled = this.isDisabledElement;
    const chkbox = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    chkbox.readonly = false;
    criteriaTitleObject.text = '- Marque apenas campos de não conformidade:';
    Http.post('/getConformities', {
      NRATIVIDADECHK: component.name.substr(8),
      NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
    }).then((response: IDictionary) => {
      response.data.dataset.criterias.forEach((criterias: IDictionary) => {
        if (criterias.IDSITUACAOGERCHK === '3' || criterias.IDSITUACAOAGEND !== '4') {
          this.desactivateButtons();
        } else if (criterias.IDSITUACAOGERCHK !== '3' && criterias.IDSITUACAOAGEND !== '4') {
          this.desactivateButtons();
        }
        btnObject.disabled = false;

        if (chkbox.datasource instanceof SimpleDatasource) {
          chkbox!.datasource!.items = criterias.CRITERIOS;
        }

        if (Object.keys(this.dataBase).includes(component.name)) {
          chkbox.value = this.dataBase[component.name].valuesChkbox.replaceAll('\'', '').split(',');
          Metadata.getInstance<Textarea>('inspectionDtNote').value = this.dataBase[component.name].valueObservation;
          Metadata.getInstance<Switch>('basicUsageSwitch').value = this.dataBase[component.name].valueSwitch;
          this.hasCriterion();
        } else {
          chkbox.value = criterias.CRITERIOSATIVOS;
          Metadata.getInstance<Textarea>('inspectionDtNote').value = criterias.DSOBSERVACAO;
          Metadata.getInstance<Switch>('basicUsageSwitch').value = false;
          this.hasCriterion();
        }
      });
    });
  }

  // Método para verificar o switch e ativar e desativar os botões
  public hasCriterion() {
    const swtch = Metadata.getInstance<Switch>('basicUsageSwitch');
    if (swtch.value === true) {
      Metadata.getInstance<Textarea>('inspectionDtNote').value = 'Vistoria Realizada, sem não conformidades.';
      Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').readonly = true;
      Metadata.getInstance<Button>('cancelInspectionDt').disabled = false;
      Metadata.getInstance<Button>('saveInspectionDt').disabled = false;
      Metadata.getInstance<Button>('newCriterioinspectionDt').disabled = true;
      Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').value = [];
    } else if (swtch.value === false) {
      Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').readonly = false;
    }
  }

  // Mostra na tela as informações da vistoria
  public setInspectionToUpdate() {
    const routeParams = getRouteParams();
    const { id } = routeParams;

    Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text = id;

    this.getInspectionNameAndLocal(id);
    this.getActivities(id);
  }

  public noPermissonModify() {
    const chkbox = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const tArea = Metadata.getInstance<Textarea>('inspectionDtNote');
    const saveButton = Metadata.getInstance<Button>('saveInspectionDt');
    const newCriteria = Metadata.getInstance<Button>('newCriterioinspectionDt');
    const fileInput = Metadata.getInstance<FileInput>('inspectionDtImage');

    chkbox.readonly = true;
    tArea.readonly = true;
    saveButton.disabled = true;
    fileInput.disabled = true;
    newCriteria.disabled = true;
  }

  // Salva a avaliação do usuário e só libera a opção de Finalizar após vistoriar todos os locais.
  public saveCriterias() {
    this.dataBase[this.selected.name] = {
      NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
      NRATIVIDADECHK: this.selected.name.substr(8),
      valuesChkbox: Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.map((dados: any) => `${dados}`).join(','),
      valueObservation: Metadata.getInstance<Textarea>('inspectionDtNote').value,
      valueSwitch: Metadata.getInstance<Switch>('basicUsageSwitch').value,
    };
    if (Object.keys(this.dataBase).length === this.quantidadeAtividades) {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Atividade avaliada com sucesso!',
        color: 'success',
        top: true,
        left: false,
        timeout: 1500,
      });
      Metadata.getInstance<Button>('FinalizeInspectionButton').disabled = false;
    }
  }

  // Salva as informações de critérios e observação para a atividade selecionada
  // public saveInspectionDetails({ component }: any) {
  //   let idSituation: any;
  //   let message: any;

  //   const params = {
  //     NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
  //     DSOBSTAREFAGERCHK: 'Vistoria realizada sem conformidades',
  //     IDSITUACAOGERCHK: '3',
  //     // NRFOTOTAREFAGERCHK: Metadata.getInstance<FileInput>('inspectionDtImage').value,
  //   };

  //   if (component.name === 'saveInspectionDt') {
  //     idSituation = 2;
  //     message = 'Alterações salvas com sucesso!';
  //     Metadata.getInstance<Button>('FinalizeInspectionButton').disabled = false;
  //   } else {
  //     idSituation = 3;
  //     message = 'Vistoria realizada com sucesso!';
  //   }

  //   Http.post('/updateInspection', [idSituation, params]).then((response: IDictionary) => {
  //     const returnMessage = response.data.dataset.info[0];
  //     if (returnMessage !== 'OK') {
  //       DialogService.show({
  //         name: 'warning-dialog',
  //         type: 'warning',
  //         title: 'Erro!',
  //         text: 'Operação bloqueada! Ocorreu um erro ao salvar os dados desta vistoria.<br><br>Se o problema persistir entre em contato com o suporte.',
  //       });
  //     } else if (returnMessage === 'OK') {
  //       Metadata.getInstance<Button>('saveInspectionDt').disabled = true;
  //       Metadata.getInstance<Button>('cancelInspectionDt').disabled = true;

  //       AlertService.show({
  //         name: 'buttons-alert',
  //         text: message,
  //         color: 'success',
  //         top: true,
  //         left: false,
  //         timeout: 1500,
  //       });
  //     }
  //   });
  // }

  // Ativa os botoões de salvar e cancelar
  public activateButtons() {
    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = false;
    cancelButtonObject.disabled = false;
  }

  // Desativa os botoões de salvar e cancelar
  public desactivateButtons() {
    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = true;
    cancelButtonObject.disabled = true;
  }

  // Limpa o dados da tela
  public clearInspectionData() {
    const chkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const textAreaObject = Metadata.getInstance<Textarea>('inspectionDtNote');
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage');
    const cardTitleObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const criteriaTitleObject = Metadata.getInstance<Text>('inspectionDtCriteriaTitle');
    const newCriterioObject = Metadata.getInstance<Button>('newCriterioinspectionDt');

    chkObject!.datasource!.data = [];
    textAreaObject.value = null;
    textAreaObject.disabled = true;
    fileInputObject.value = true;
    fileInputObject.disabled = true;
    cardTitleObject.subtitle = '';
    criteriaTitleObject.text = '- Nenhuma atividade selecionada.';
    newCriterioObject.disabled = true;

    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = true;
    cancelButtonObject.disabled = true;

    this.clearActivitiesColor();
  }

  // Tira a seleção dos cards de atividade
  public clearActivitiesColor() {
    const iterableCardObject = Metadata.getInstance<IterableCarousel>('inspectionDtActivities');
    iterableCardObject.children.forEach((activity: IDictionary) => {
      activity.color = '#ffffff';
    });
  }

  // Volta para tela de vistorias
  public finishInspectionDetails() {
    Router.replace('/vistorias');
  }

  // Confirma se quer cancelar
  public confirmCancelButton() {
    DialogService.show({
      name: 'cancelButton',
      type: 'warning',
      title: 'Cancelar',
      text: 'Deseja cancelar suas alterações?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.dialogConfirmEvents,
        },
      ],
    });
  }

  // Confirma se quer salvar
  public confirmSaveButton() {
    DialogService.show({
      name: 'cancelButton',
      type: 'warning',
      title: 'Salvar',
      text: 'Deseja salvar suas alterações para esta atividade?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
        },
      ],
    });
  }

  // Confirma se quer deixar a página
  public confirmFinishButton() {
    DialogService.show({
      name: 'finishButton',
      type: 'warning',
      title: 'Concluir',
      text: 'Todas as alterações não salvas serão perdidas. Você deseja mesmo continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.finishConfirmEvents,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
        },
      ],
    });
  }

  // Tratamento do modal para o botão "novo critério"
  public createModal() {
    this.modalController.createBasicUsageModal();
  }

  public destroyModal() {
    this.modalController.destroyBasicUsageModal();
  }

  public openModal() {
    Promise.all([this.modalController.openBasicModal()]).then(() => {
      const buttonObject = Metadata.getInstance<Button>('closeModalButton');
      buttonObject.events = {
        click: () => this.newCriterio(),
      };
    });
  }

  public fecharModal() {
    this.modalController.closeBasicModal();
  }

  // Adiciona um novo critério
  public newCriterio() {
    const form = Metadata.getInstance<Form>('modalCriacaoCriterio');
    if (form.validate()) {
      const chkObject = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
      const criterioName = Metadata.getInstance<TextInput>('InputNovoCriterio').value;
      chkObject!.datasource!.data!.push({
        name: criterioName,
        value: null,
      });
      this.fecharModal();
    }
  }

  // Teste Adicionar foto
  public updloadPictures() {
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage').value;
    const arquivos: IDictionary = [];
    const arquivo = {
      FOTO: [],
    };
    const types: IDictionary = ['jpg', 'jpeg', 'png'];

    fileInputObject.forEach((database: IDictionary) => {
      const typeSplited = database.type.split('/', 2);
      const params = {
        name: database.name,
        type: typeSplited[1],
        size: database.size,
      };
      const verifica = types.filter((type:string) => type === params.type);
      if (verifica.length > 0) {
        arquivos.push(params);
      }
    });
    arquivo.FOTO = arquivos as any;
    // Http.post('/uploadPictures', { arquivo });
  }

  public consoleLog(variavel: any) {
    const titulo = Metadata.getInstance<Text>('titulo');
    titulo.text = variavel;
  }
}
